import os
import torch
import numpy as np
from segment_anything import sam_model_registry, SamAutomaticMaskGenerator, SamPredictor
import matplotlib.pyplot as plt
import supervision as sv
from PIL import Image
import torchvision.transforms as T
from pypher.pypher import psf2otf
from skimage.filters import gaussian
from numpy.fft import fft2, ifft2, fftshift, ifftshift

CHECKPOINT_PATH = "/home/irmak/convex_nerf/sam/sam_vit_h_4b8939.pth"

device = torch.device('cuda:0' if torch.cuda.is_available() else 'cpu')
MODEL_TYPE = "vit_h"
sam = sam_model_registry[MODEL_TYPE](checkpoint=CHECKPOINT_PATH).to(device=device)
mask_generator = SamAutomaticMaskGenerator(sam)

data = np.load('/home/irmak/convex_nerf/tiny_nerf_data.npz')

n_training = 100
testimg_idx = 101


images = data['images'][:n_training]
testimg = data['images'][testimg_idx]
print(images.shape)
print(testimg.shape)
height, width = images.shape[1:3]


def add_noise(x, sigma):
    return x + np.random.randn(x.shape[0],x.shape[1], x.shape[2]) * sigma

def fspecial_gaussian_2d(size, sigma):
    kernel = np.zeros(tuple(size))
    kernel[size[0]//2, size[1]//2] = 1
    kernel = gaussian(kernel, sigma)
    return kernel/np.sum(kernel)

def add_blur(img, sigma):
    out = np.zeros_like(img)
    for i in range(img.shape[-1]):
        im = img[:,:,i]
        imgFFT = fftshift(fft2(im))
        filtSize = np.ceil(9 * sigma).astype(int)
        lp = fspecial_gaussian_2d((filtSize, filtSize), sigma)

        otf = fftshift(psf2otf(psf=lp, shape=(height, width)))
        out[:,:,i] = np.real(ifft2(ifftshift(otf * imgFFT)))

    return out

def get_masks(img,savedir,i):

    sam_result = mask_generator.generate(img)

    masks = [
        mask['segmentation']
        for mask
        in sorted(sam_result, key=lambda x: x['area'], reverse=True)
    ]
    print(len(masks))
    plt.imshow(masks[0])
    plt.savefig(f"{savedir}/mask_img_{i}")
    return masks[0]

# sigmas = [0.05, 0.1, 0.2] 
# for sigma in sigmas:
#     savedir = "/home/irmak/convex_nerf/noisy_sam/noisy_masks_sigma_{}".format(sigma)
#     os.makedirs(savedir, exist_ok=True)
#     noisy_train_masks = []
#     for i in range(images.shape[0]):
#         img = images[i]
#         noisy_img = add_noise(img,sigma)
#         mask = get_masks(noisy_img, savedir, i)
#         noisy_train_masks.append(mask)

#     noisy_test = add_noise(testimg,sigma)
#     noisy_test_mask = get_masks(noisy_test, savedir, i=101)
#     np.save(savedir + "noisy_train.npy", noisy_train_masks)
#     np.save(savedir + "noisy_test.npy", noisy_test_mask)


sigmas = [0.5, 0.75, 0.9] #1 works
for sigma in sigmas:
    savedir = "/home/irmak/convex_nerf/noisy_sam/blurry_masks_sigma_{}".format(sigma)
    os.makedirs(savedir, exist_ok=True)
    noisy_train_masks = []
    for i in range(images.shape[0]):
        img = images[i]
        noisy_img = add_blur(img,sigma)
        mask = get_masks(noisy_img, savedir, i)
        noisy_train_masks.append(mask)

    noisy_test = add_blur(testimg,sigma)
    noisy_test_mask = get_masks(noisy_test, savedir, i=101)
    np.save(savedir + "/blurry_train.npy", noisy_train_masks)
    np.save(savedir + "/blurry_test.npy", noisy_test_mask)


# for i,m in enumerate(masks):
#     plt.imshow(m)
#     plt.savefig(f"mask_{i}")